﻿using ThinkGeo.MapSuite.Core;
using System;
using System.IO;

namespace OledbFeatureSource
{
    // The FeatureLayer is a wrapper to the FeatureSource and provides all of the drawing specific
    // methods such as the ZoomLevels and Styles.  As you can see it is just a simple wrapper and the
    // real work is already done for you in the FeatureLayer base class.
    public class OledbFeatureLayer : FeatureLayer
    {
        OledbFeatureSource oledbFeatureSource;

        // It is important for compatability that you always have a default constructor that
        // takes no parameters.  This constructor will just chain to the more complex one.
        public OledbFeatureLayer()
            : this(string.Empty, string.Empty, string.Empty, string.Empty, string.Empty, string.Empty, string.Empty, string.Empty)
        { }

        // We have provided another constructor that has all of the parameters we need for the FeatureSource
        // to work.
        public OledbFeatureLayer(string tableName, string idColumnName, string wellKnownTextColumnName, string boundaryUpperLeftXColumnName, string boundaryUpperLeftYColumnName, string boundaryLowerRightXColumnName, string boundaryLowerRightYColumnName, string connectionString)
            : base()
        {
            oledbFeatureSource = new OledbFeatureSource();
            this.FeatureSource = oledbFeatureSource;

            TableName = tableName;
            IdColumnName = idColumnName;
            WellKnownBinaryColumnName = wellKnownTextColumnName;

            BoundaryUpperLeftXColumnName = boundaryUpperLeftXColumnName;
            BoundaryUpperLeftYColumnName = boundaryUpperLeftYColumnName;
            BoundaryLowerRightXColumnName = boundaryLowerRightXColumnName;
            BoundaryLowerRightYColumnName = boundaryLowerRightYColumnName;

            ConnectionString = connectionString;
            int index = connectionString.IndexOf("data source=", StringComparison.CurrentCultureIgnoreCase);
            if (index > 0)
            {
                int semicolon = connectionString.IndexOf(";", index);
                string datasource = connectionString.Substring(index + "data source=".Length, semicolon - index - "data source=".Length);

                string dirName = Path.GetDirectoryName(datasource);

                IndexFileName = dirName + "\\" + tableName + ".idx";
            }
        }

        // The next dozen lines are all of the properties we need.  It is good form to always match
        // the parameters in your constructors with properties of the exact same name.
        public string ConnectionString
        {
            get { return oledbFeatureSource.ConnectionString; }
            set { oledbFeatureSource.ConnectionString = value; }
        }

        public bool RequireIndex
        {
            get { return oledbFeatureSource.RequireIndex; }
            set { oledbFeatureSource.RequireIndex = value; }
        }

        public string TableName
        {
            get { return oledbFeatureSource.TableName; }
            set { oledbFeatureSource.TableName = value; }
        }

        public string IdColumnName
        {
            get { return oledbFeatureSource.IdColumnName; }
            set { oledbFeatureSource.IdColumnName = value; }
        }

        public string WellKnownBinaryColumnName
        {
            get { return oledbFeatureSource.WellKnownBinaryColumnName; }
            set { oledbFeatureSource.WellKnownBinaryColumnName = value; }
        }

        public string BoundaryUpperLeftXColumnName
        {
            get { return oledbFeatureSource.BoundaryUpperLeftXColumnName; }
            set { oledbFeatureSource.BoundaryUpperLeftXColumnName = value; }
        }

        public string BoundaryUpperLeftYColumnName
        {
            get { return oledbFeatureSource.BoundaryUpperLeftYColumnName; }
            set { oledbFeatureSource.BoundaryUpperLeftYColumnName = value; }
        }

        public string BoundaryLowerRightXColumnName
        {
            get { return oledbFeatureSource.BoundaryLowerRightXColumnName; }
            set { oledbFeatureSource.BoundaryLowerRightXColumnName = value; }
        }

        public string BoundaryLowerRightYColumnName
        {
            get { return oledbFeatureSource.BoundaryLowerRightYColumnName; }
            set { oledbFeatureSource.BoundaryLowerRightYColumnName = value; }
        }

        public static void BuildIndex(string connectionString, string tableName, string idColumnName, string wellKnownBinaryColumnName, BuildIndexMode buildIndexMode)
        {
            OledbFeatureSource.BuildIndex(connectionString, tableName, idColumnName, wellKnownBinaryColumnName, buildIndexMode);
        }

        public string IndexFileName
        {
            get { return oledbFeatureSource.IndexFileName; }
            set { oledbFeatureSource.IndexFileName = value; }
        }
    }
}
