﻿using System;
using System.Collections.ObjectModel;
using System.Drawing;
using System.Windows.Forms;
using ThinkGeo.MapSuite.Core;

namespace OledbFeatureSource
{
    public partial class TestForm : Form
    {
        private MapEngine mapEngine = new MapEngine();
        private Bitmap bitmap;

        public TestForm()
        {
            InitializeComponent();
        }

        private void TestForm_Load(object sender, System.EventArgs e)
        {
            // Set the current extent and background color
            mapEngine.CurrentExtent = ExtentHelper.GetDrawingExtent(new RectangleShape(-180.0, 83.0, 180.0, -90.0), picMap.Width, picMap.Height);
            mapEngine.BackgroundFillBrush = new GeoSolidBrush(GeoColor.GeographicColors.DeepOcean);
            
            // Display OledbFeatureLayer
            string connectionString = @"Provider=Microsoft.Jet.OLEDB.4.0;Data Source=..\..\data\Countries.mdb;User Id=admin;Password=;";
            OledbFeatureLayer.BuildIndex(connectionString, "Countries", "ID", "WellKnownBinary", BuildIndexMode.DoNotRebuild);
            OledbFeatureLayer countriesLayer = new OledbFeatureLayer("Countries", "ID", "WellKnownBinary", "BoundaryUpperLeftX", "BoundaryUpperLeftY", "BoundaryLowerRightX", "BoundaryLowerRightY", connectionString);

            countriesLayer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle = AreaStyles.Country1;
            countriesLayer.ZoomLevelSet.ZoomLevel01.DefaultTextStyle = TextStyles.City1("CountryName");
            countriesLayer.ZoomLevelSet.ZoomLevel01.DefaultTextStyle.FittingPolygon = true;
            countriesLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

            // Add the new layer to the MapEngine
            mapEngine.StaticLayers.Add("CountriesLayer", countriesLayer);
            DrawImage();
        }

        private void btnEdit_Click(object sender, System.EventArgs e)
        {
            FeatureLayer countriesLayer = (FeatureLayer)mapEngine.StaticLayers["CountriesLayer"];
            if (Validator.CheckMdbHasRecords((OledbFeatureSource)countriesLayer.FeatureSource))
            {
                // Edit the shape itself
                countriesLayer.Open();
                Collection<Feature> features = countriesLayer.FeatureSource.GetAllFeatures(ReturningColumnsType.AllColumns);
                countriesLayer.EditTools.BeginTransaction();

                for (int i = 0; i < features.Count; i++)
                {
                    // Change the country name
                    features[i].ColumnValues["CountryName"] += "(Modified)";
                    // Scale down the country
                    BaseShape baseShape = features[i].GetShape();
                    baseShape.ScaleTo(0.5);
                    byte[] bytes = baseShape.GetWellKnownBinary();
                    Feature currentFeature = features[i];
                    currentFeature = new Feature(bytes, features[i].Id, features[i].ColumnValues);
                    //Array.Copy(bytes, features[i].WellKnownBinary, bytes.Length);
                    countriesLayer.EditTools.Update(features[i]);
                }
                countriesLayer.EditTools.CommitTransaction();
                countriesLayer.Close();

                DrawImage();
            }
            else
            {
                MessageBox.Show("No records in the specified .mdb file, please check it.");
            }
        }

        private void btnDelete_Click(object sender, System.EventArgs e)
        {
            FeatureLayer countriesLayer = (FeatureLayer)mapEngine.StaticLayers["CountriesLayer"];

            countriesLayer.Open();
            countriesLayer.EditTools.BeginTransaction();
            Collection<Feature> features = countriesLayer.FeatureSource.GetAllFeatures(ReturningColumnsType.AllColumns);
            foreach (Feature feature in features)
            {
                countriesLayer.EditTools.Delete(feature.Id);
            }
            countriesLayer.EditTools.CommitTransaction();
            countriesLayer.Close();

            DrawImage();
        }

        private void btnImport_Click(object sender, System.EventArgs e)
        {
            FeatureLayer countriesLayer = (FeatureLayer)mapEngine.StaticLayers["CountriesLayer"];
            if (!Validator.CheckMdbHasRecords((OledbFeatureSource)countriesLayer.FeatureSource))
            {
                // Import the records of shape file into .mdb file
                string shpPath = @"..\..\Data\Countries02.shp";
                ShapeFileFeatureSource worldShpFeatureSource = new ShapeFileFeatureSource(shpPath, ShapeFileReadWriteMode.ReadOnly);
                worldShpFeatureSource.Open();
                Collection<Feature> features = worldShpFeatureSource.GetAllFeatures(ReturningColumnsType.AllColumns);
                worldShpFeatureSource.Close();

                countriesLayer.Open();
                countriesLayer.EditTools.BeginTransaction();
                foreach (Feature feature in features)
                {
                    string countryName = feature.ColumnValues["CNTRY_NAME"].ToString();
                    feature.ColumnValues.Clear();
                    feature.ColumnValues.Add("CountryName", countryName);
                    countriesLayer.EditTools.Add(feature);
                }
                countriesLayer.EditTools.CommitTransaction();
                countriesLayer.Close();
            }
            else
            {
                MessageBox.Show("Failed to import the data, please make sure there is't any records in the specified .mdb file.");
            }
            DrawImage();
        }

        private void btnGetCount_Click(object sender, EventArgs e)
        {
            FeatureLayer countriesLayer = (FeatureLayer)mapEngine.StaticLayers["CountriesLayer"];
            countriesLayer.Open();
            int featureCount = countriesLayer.FeatureSource.GetCount();
            MessageBox.Show("Number of Countries: " + featureCount, "Get Count");
            countriesLayer.Close();
        }

        private void btnClose_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void DrawImage()
        {
            if (bitmap != null)
            {
                bitmap.Dispose();
            }
            bitmap = new Bitmap(picMap.Width, picMap.Height);
            mapEngine.OpenAllLayers();
            mapEngine.DrawStaticLayers(bitmap, GeographyUnit.DecimalDegree);
            //mapEngine.CloseAllLayers();
            picMap.Image = bitmap;
        }

        private void toolBar_ButtonClick(object sender, ToolBarButtonClickEventArgs e)
        {
            switch (e.Button.Tag.ToString())
            {
                case "Zoom In":
                    mapEngine.CurrentExtent.ScaleDown(50);
                    break;
                case "Zoom Out":
                    mapEngine.CurrentExtent.ScaleUp(50);
                    break;
                case "Full Extent":
                    mapEngine.CurrentExtent = ExtentHelper.GetDrawingExtent(new RectangleShape(-180.0, 83.0, 180.0, -90.0), picMap.Width, picMap.Height);
                    break;
                case "Pan Left":
                    mapEngine.CurrentExtent = ExtentHelper.Pan(mapEngine.CurrentExtent, PanDirection.Left, 20);
                    break;
                case "Pan Right":
                    mapEngine.CurrentExtent = ExtentHelper.Pan(mapEngine.CurrentExtent, PanDirection.Right, 20);
                    break;
                case "Pan Up":
                    mapEngine.CurrentExtent = ExtentHelper.Pan(mapEngine.CurrentExtent, PanDirection.Up, 20);
                    break;
                case "Pan Down":
                    mapEngine.CurrentExtent = ExtentHelper.Pan(mapEngine.CurrentExtent, PanDirection.Down, 20);
                    break;
                default:
                    break;
            }
            DrawImage();
        }
    }
}
